local L = AceLibrary("AceLocale-2.2"):new("CurrencyFu")
local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local endColor = "|r"
local newLine = "\n"

CurrencyFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
CurrencyFu.hasIcon = true
CurrencyFu.independentProfile = true
CurrencyFu.defaultPosition = "LEFT"
CurrencyFu.defaultMinimapPosition = 180
CurrencyFu.cannotDetachTooltip = true
CurrencyFu.tooltipType = "Tablet-2.0"
CurrencyFu.clickableTooltip = true
CurrencyFu:RegisterDB("CurrencyFuDB", "CurrencyFuDBPC")

CurrencyFu:RegisterDefaults("char", {new = 0, total = 0})
CurrencyFu:RegisterDefaults("profile", {
    hidemyloot = false,
	hideotherloot = false,
	showemblems = true,
	showtokens = true,
	showmarks = true,
	showother = true,
	listemblems = true,
	listtokens = true,
	listmarks = true,
	listother = true,
	selection = {
	    emblemofconquest = false,
		emblemofvalor = false,
		emblemofheroism = false,
		badgeofjustice = false,
		dalarancookingaward = false,
		dalaranjewelcrafterstoken = false,
		alteracvalley = false,
		arathibasin = false,
		eyeofthestorm = false,
		strandoftheancients = false,
		warsonggulch = false,
		wintergrasp = false,
		stonekeepersshard = false,
		championsseal = false,
		venturecoin = false,
	},
})

local defIgnoredLoot = {
	"Emblem of Conquest",
	"Emblem of Valor",
	"Emblem of Heroism",
	"Badge of Justice",
	"Dalaran Cooking Award",
	"Dalaran Jewelcrafter's Token",
	"Alterac Valley Mark of Honor",
	"Arathi Basin Mark of Honor",
	"Eye of the Storm Mark of Honor",
	"Strand of the Ancients Mark of Honor",
	"Warsong Gulch Mark of Honor",
	"Wintergrasp Mark of Honor",
	"Stone Keeper's Shard",
	"Champion's Seal",
	"Venture Coin",
}

local selectionTable = {
	"emblemofconquest",
	"emblemofvalor",
	"emblemofheroism",
	"badgeofjustice",
	"dalarancookingaward",
	"dalaranjewelcrafterstoken",
	"alteracvalley",
	"arathibasin",
	"eyeofthestorm",
	"strandoftheancients",
	"warsonggulch",
	"wintergrasp",
	"stonekeepersshard",
	"championsseal",
	"venturecoin",
}

local selShortName = {
	"EoC",
	"EoV",
	"EoH",
	"BoJ",
	"DCA",
	"DJT",
	"AV",
	"AB",
	"EoS",
	"SoE",
	"WSG",
	"WG",
	"SKS",
	"CS",
	"VC",
}
	
local selectionIds = {
	45624,
	40753,
	40752,
	29434,
	43016,
	41596,
	20560,
	20559,
	29024,
	42425,
	20558,
	43589,
	43228,
	44990,
	37836,
}

local defIgnoredChannels = {
	"SAY",
	"PARTY",
	"RAID",
	"GUILD",
	"CHANNEL",
	"LOOT",
}

local itemTable = {
	Emblems = {
		45624,
		40753,
		40752,
		29434,
	},
	Tokens = {
		43016,
		41596,
	},
	Marks = {
		20560,
		20559,
		29024,
		42425,
		20558,
		43589,
	},
	Others = {
		43228,
		44990,
		37836,
	},
}

function CurrencyFu:OnMenuRequest(level, value, inTooltip)
	if (level == 1) then
	    dewdrop:AddLine(
     		'text', "Open Currency",
   			'tooltipTitle', "Open Currency",
            'tooltipText', "Open main Currency window",
            'func', function() ToggleCharacter("TokenFrame"); end
		)
		dewdrop:AddLine()
		dewdrop:AddLine(
	  		'text', "Loot text",
	  		'tooltipTitle', "Loot text",
			'tooltipText', "Show/Hide currency loot text in chat",
			'hasArrow', true,
			'value', "loot"
		)
  		dewdrop:AddLine(
		    'text', "Categories",
		    'tooltipTitle', "Categories",
		    'tooltipText', "Show/Hide currency categories in tooltip",
		    'hasArrow', true,
		    'value', "categories"
		)
		dewdrop:AddLine(
			'text', "Text selection",
			'tooltipTitle', "Text selection",
			'tooltipText', "Selection of currencies to show in FuBar text",
			'hasArrow', true,
			'value', "selection"
		)
		dewdrop:AddLine()
    elseif (level == 2) then
        if (value == "loot") then
            dewdrop:AddLine(
                'text', "Hide my loot",
                'tooltipTitle', "Hide my loot",
                'tooltipText', "Hides only your loot chat.",
                'checked', self.db.profile.hidemyloot,
                'func', function() self:ToggleHideMyLoot() end
			)
			dewdrop:AddLine(
			    'text', "Hide other loot",
			    'tooltipTitle', "Hide other loot",
			    'tooltipText', "Hides raid/party/guild loot chat.",
			    'checked', self.db.profile.hideotherloot,
			    'func', function() self:ToggleHideOtherLoot() end
			)
	    elseif (value == "categories") then
			dewdrop:AddLine(
	  			'text', "Emblems and Badges",
	  			'tooltipTitle', "Emblems and Badges",
				'tooltipText', "Show Emblems and Badges in tooltip",
				'checked', self.db.profile.showemblems,
				'func', function() self:ToggleShowEmblems() end
			)
			dewdrop:AddLine(
	  			'text', "Tokens",
	  			'tooltipTitle', "Tokens",
				'tooltipText', "Show Tokens in tooltip",
				'checked', self.db.profile.showtokens,
				'func', function() self:ToggleShowTokens() end
			)
			dewdrop:AddLine(
	  			'text', "Marks of Honor",
	  			'tooltipTitle', "Marks of Honor",
				'tooltipText', "Show Marks of Honor in tooltip",
				'checked', self.db.profile.showmarks,
				'func', function() self:ToggleShowMarks() end
			)
			dewdrop:AddLine(
	  			'text', "Other",
	  			'tooltipTitle', "Other",
				'tooltipText', "Show Other in tooltip",
				'checked', self.db.profile.showother,
				'func', function() self:ToggleShowOther() end
			)
	    elseif (value == "selection") then
	        for i, n in pairs(selectionTable) do
				dewdrop:AddLine(
			        'text', defIgnoredLoot[i],
			        'tooltipTitle', defIgnoredLoot[i],
			        'tooltipText', "Show/Hide "..defIgnoredLoot[i],
			        'checked', self.db.profile.selection[n],
			        'func', function() self:ToggleText(n) end
				)
			end
		end
	end
end

function chatFilter(self, event, ...)

	local msg = ...
	
	local hideMyLoot = CurrencyFu:IsHideMyLoot()
	local hideOtherLoot = CurrencyFu:IsHideOtherLoot()
	
	local findMsgYou = string.find(msg, "^You")
	local findMsg1 = string.find(msg, "receive")
	local findMsg2 = string.find(msg, "create")

	if hideMyLoot then
		if findMsgYou ~= nil then
			if findMsg1 ~= nil or findMsg2 ~= nil then
				for i, e in pairs(defIgnoredLoot) do
				    if defIgnoredLoot[i] ~= nil then
						if string.find(msg, defIgnoredLoot[i]) ~= nil then
							return true
						end
					end
				end
			end
		end
	end
	if hideOtherLoot then
		if findMsgYou == nil then
			if findMsg1 ~= nil or findMsg2 ~= nil then
				for i, e in pairs(defIgnoredLoot) do
				    if defIgnoredLoot[i] ~= nil then
						if string.find(msg, defIgnoredLoot[i]) ~= nil then
							return true
						end
					end
				end
			end
		end
	end
end

function CurrencyFu:OnInitialize()

	self:SetIcon("Interface\\AddOns\\FuBar_CurrencyFu\\icon.tga")
	
	ignoredLoot = defIgnoredLoot

	if self.db.profile.hidemyloot then
	    for i, v in pairs(defIgnoredChannels) do
	    	ChatFrame_AddMessageEventFilter("CHAT_MSG_"..v, chatFilter)
		end
	    self:Print(("Loot chat from my loot is |cffff0000Hidden.|r"))
	else
	    for i, v in pairs(defIgnoredChannels) do
	    	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_"..v, chatFilter)
		end
	    self:Print(("Loot chat from my loot is |cff00ff00Visible.|r"))
	end
	
	if self.db.profile.hideotherloot then
 	    for i, v in pairs(defIgnoredChannels) do
			ChatFrame_AddMessageEventFilter("CHAT_MSG_"..v, chatFilter)
		end
		self:Print(("Loot chat from raid/party/guild is |cffff0000Hidden.|r"))
	else
	    for i, v in pairs(defIgnoredChannels) do
	    	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_"..v, chatFilter)
		end
		self:Print(("Loot chat from raid/party/guild is |cff00ff00Visible.|r"))
	end

end

function CurrencyFu:OnLoad()

end

function CurrencyFu:OnEnable()

    self:RegisterEvent("PLAYER_MONEY")
	
	self:OnTextUpdate()
	
end

function CurrencyFu:OnDisable()

    self:UnregisterEvent("PLAYER_MONEY")

end

function CurrencyFu:PLAYER_MONEY(event)

	self:OnTextUpdate()
	
end

function CurrencyFu:OnTextUpdate()

	if self:IsTextShown() then
	    local text = " "
	    local count
	    for i, n in pairs(selectionTable) do
	        if self.db.profile.selection[n] then
	            count = GetItemCount(selectionIds[i])
	            if count ~= 0 and count ~= nil then
	            	text = text.."|cffffffff"..selShortName[i]..":|r "..count.." "
				else
			    	text = text.."|cffffffff"..selShortName[i]..":|r 0 "
				end
			end
		end
		self:SetText(text)
    else
    	self:HideText()
    end
    
    if not self:IsIconShown() then
        self:HideIcon()
    end

end

function CurrencyFu:OnClick(button)

	if button == "LeftButton" then
		ToggleCharacter("TokenFrame")
	end

end

local isDown = nil

function CurrencyFu:GetDropEmblems()
	if self.db.profile.listemblems then
		self.db.profile.listemblems = false
	else
		self.db.profile.listemblems = true
	end
	self:Update()
end

function CurrencyFu:GetDropTokens()
	if self.db.profile.listtokens then
		self.db.profile.listtokens = false
	else
		self.db.profile.listtokens = true
	end
	self:Update()
end

function CurrencyFu:GetDropMarks()
	if self.db.profile.listmarks then
		self.db.profile.listmarks = false
	else
		self.db.profile.listmarks = true
	end
	self:Update()
end

function CurrencyFu:GetDropOther()
	if self.db.profile.listother then
		self.db.profile.listother = false
	else
		self.db.profile.listother = true
	end
	self:Update()
end

function CurrencyFu:OnTooltipUpdate()
	
	local supercat = tablet:AddCategory(
	'columns', 2
	)
	cat = supercat:AddLine(
	'text', " "
	)
	if (self.db.profile.showemblems) then
		cat = supercat:AddCategory(
		'text', " ",
		'hideBlankLine', true
		)
		cat:AddLine(
		'text', "|cffffffffEmblems and Badges|r",
		'size', 14,
		'text2', " ",
		'size2', 14,
		'arg1', self,
		'func', "GetDropEmblems",
		'arg2', self.db.profile.listemblems
		)
		if (self.db.profile.listemblems) then
		    cat:AddCategory(
			'text', " ",
			'columns', 2,
			'hideBlankLine', true
			)
			for a, s in pairs(itemTable.Emblems) do
				local name, link, _, _, _, _, _, _, _, texture = GetItemInfo(s)
				local count = GetItemCount(s)
				if name ~= nil then
					if count ~= 0 and count ~= nil then
		     			cat:AddLine(
						'text', " "..name,
						'text2', count,
						'hasCheck', true,
						'checked', true,
						'checkIcon', texture,
						'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": "..count,
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
		    			)
		    		else
		    			cat:AddLine(
		    			'text', " "..name,
		    			'text2', "-",
		    			'hasCheck', true,
		    			'checked', true,
		    			'checkIcon', texture,
		    			'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": -",
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
		    			)
		    		end
		    	end
			end
		end
	end

	if (self.db.profile.showtokens) then
		cat = supercat:AddCategory(
		'text', " ",
  		'hideBlankLine', true
		)
		cat:AddLine(
		'text', "|cffffffffTokens|r",
		'size', 14,
		'text2', " ",
		'size2', 14,
		'arg1', self,
		'func', "GetDropTokens",
		'arg2', self.db.profile.listtokens
		)
		if (self.db.profile.listtokens) then
		    cat:AddCategory(
			'text', " ",
			'columns', 2,
			'hideBlankLine', true
			)
			for a, s in pairs(itemTable.Tokens) do
				local name, link, _, _, _, _, _, _, _, texture = GetItemInfo(s)
				local count = GetItemCount(s)
				if name ~= nil then
					if count ~= 0 and count ~= nil then
						cat:AddLine(
						'text', " "..name,
						'text2', count,
						'hasCheck', true,
						'checked', true,
						'checkIcon', texture,
						'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": "..count,
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
						)
					else
						cat:AddLine(
						'text', " "..name,
						'text2', "-",
						'hasCheck', true,
						'checked', true,
						'checkIcon', texture,
						'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": -",
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
						)
					end
				end
			end
		end
	end

	if (self.db.profile.showmarks) then
		cat = supercat:AddCategory(
		'text', " ",
  		'hideBlankLine', true
		)
		cat:AddLine(
		'text', "|cffffffffMarks of Honor|r",
		'size', 14,
		'text2', " ",
		'size2', 14,
		'arg1', self,
		'func', "GetDropMarks",
		'arg2', self.db.profile.listmarks
		)
		if (self.db.profile.listmarks) then
		    cat:AddCategory(
			'text', " ",
			'columns', 2,
			'hideBlankLine', true
			)
			for a, s in pairs(itemTable.Marks) do
				local name, link, _, _, _, _, _, _, _, texture = GetItemInfo(s)
				local count = GetItemCount(s)
				if name ~= nil then
					if count ~= 0 and count ~= nil then
						cat:AddLine(
						'text', " "..string.gsub(name, " Mark of Honor", ""),
						'text2', count,
						'hasCheck', true,
						'checked', true,
						'checkIcon', texture,
						'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": "..count,
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
						)
					else
						cat:AddLine(
						'text', " "..string.gsub(name, " Mark of Honor", ""),
						'text2', "-",
						'hasCheck', true,
						'checked', true,
						'checkIcon', texture,
						'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": -",
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
						)
					end
				end
			end
		end
	end

	if (self.db.profile.showother) then
		cat = supercat:AddCategory(
		'text', " ",
		'hideBlankLine', true
		)
		cat:AddLine(
		'text', "|cffffffffOther|r",
		'size', 14,
		'text2', " ",
		'size2', 14,
		'arg1', self,
		'func', "GetDropOther",
		'arg2', self.db.profile.listother
		)
		if (self.db.profile.listother) then
		    cat:AddCategory(
			'text', " ",
			'columns', 2,
			'hideBlankLine', true
			)
			for a, s in pairs(itemTable.Others) do
				local name, link, _, _, _, _, _, _, _, texture = GetItemInfo(s)
				local count = GetItemCount(s)
				if name ~= nil then
					if count ~= 0 and count ~= nil then
						cat:AddLine(
						'text', " "..name,
						'text2', count,
						'hasCheck', true,
						'checked', true,
						'checkIcon', texture,
						'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": "..count,
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
						)
					else
						cat:AddLine(
						'text', " "..name,
						'text2', "-",
						'hasCheck', true,
						'checked', true,
						'checkIcon', texture,
						'func', self.TooltipClick,
						'arg1', self,
						'arg2', link..": -",
						'onEnterFunc', self.SetTooltip,
						'onEnterArg1', self,
						'onEnterArg2', link,
						'onLeaveFunc', GameTooltip.Hide,
						'onLeaveArg1', GameTooltip
						)
					end
				end
			end
		end
	end
	
	local hint = ""
	local clickColor = "|cffeda55f"
	hint = hint..newLine..format("%s to open Currency window.", clickColor.."Left click"..endColor)
			   ..newLine..format("%s to open Options menu.", clickColor.."Right click"..endColor)
			   ..newLine..format("%s on group name to expand it.", clickColor.."Left click"..endColor)
			   ..newLine..format("%s on currency name to paste it in an open chat line.", clickColor.."Left click"..endColor)

	tablet:SetHint(hint)

end

function CurrencyFu:IsHideMyLoot()
	return self.db.profile.hidemyloot
end

function CurrencyFu:ToggleHideMyLoot()
	self.db.profile.hidemyloot = not self.db.profile.hidemyloot

	if self.db.profile.hidemyloot then
	    for i, v in pairs(defIgnoredChannels) do
			ChatFrame_AddMessageEventFilter("CHAT_MSG_"..v, chatFilter)
		end
		self:Print(("Loot chat from my loot is |cffff0000Hidden.|r"))
	else
	    for i, v in pairs(defIgnoredChannels) do
	    	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_"..v, chatFilter)
	    end
	    self:Print(("Loot chat from my loot is |cff00ff00Visible.|r"))
	end

	self:Update()
end

function CurrencyFu:IsHideOtherLoot()
	return self.db.profile.hideotherloot
end

function CurrencyFu:ToggleHideOtherLoot()
	self.db.profile.hideotherloot = not self.db.profile.hideotherloot

	if self.db.profile.hideotherloot then
  		for i, v in pairs(defIgnoredChannels) do
			ChatFrame_AddMessageEventFilter("CHAT_MSG_"..v, chatFilter)
		end
		self:Print(("Loot chat from raid/party/guild is |cffff0000Hidden.|r"))
	else
	    for i, v in pairs(defIgnoredChannels) do
	    	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_"..v, chatFilter)
		end
	    self:Print(("Loot chat from raid/party/guild is |cff00ff00Visible.|r"))
	end

	self:Update()
end

function CurrencyFu:IsShowEmblems()
	return self.db.profile.showemblems
end

function CurrencyFu:ToggleShowEmblems()
	self.db.profile.showemblems = not self.db.profile.showemblems
	self:Update()
end

function CurrencyFu:IsShowTokens()
	return self.db.profile.showtokens
end

function CurrencyFu:ToggleShowTokens()
	self.db.profile.showtokens = not self.db.profile.showtokens
	self:Update()
end

function CurrencyFu:IsShowMarks()
	return self.db.profile.showmarks
end

function CurrencyFu:ToggleShowMarks()
	self.db.profile.showmarks = not self.db.profile.showmarks
	self:Update()
end

function CurrencyFu:IsShowOther()
	return self.db.profile.showother
end

function CurrencyFu:ToggleShowOther()
	self.db.profile.showother = not self.db.profile.showother
	self:Update()
end

function CurrencyFu:ToggleText(value)
	self.db.profile.selection[value] = not self.db.profile.selection[value]
	self:Update()
end

function CurrencyFu:SetTooltip(item)
    local tipFrame = GameTooltip

    GameTooltip_SetDefaultAnchor(tipFrame, this)

    tipFrame:SetHyperlink(item)

    tipFrame:Show()
end

function CurrencyFu:TooltipClick(item)
    if ChatFrameEditBox:IsVisible() then
        ChatFrameEditBox:Insert(item)
    end
end