local L = AceLibrary("AceLocale-2.2"):new("DualSpecFu")
local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local endColor = "|r"
local newLine = "\n"
local db
local groupNames = {
	"primary",
	"secondary"
}
local groupNamesCaps = {
	"Primary",
	"Secondary"
}
local className = {
	["DRUID"]      = "Druid",
	["HUNTER"]     = "Hunter",
	["MAGE"]       = "Mage",
	["PALADIN"]    = "Paladin",
	["PRIEST"]     = "Priest",
	["ROGUE"]      = "Rogue",
	["SHAMAN"]     = "Shaman",
	["WARLOCK"]    = "Warlock",
	["WARRIOR"]    = "Warrior",
	["DEATHKNIGHT"] = "Deathknight"
}

local primarySpecSpellName = GetSpellInfo(63645)
local secondarySpecSpellName = GetSpellInfo(63644)
local specCache = {}
local iconCache = {}

DualSpecFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0","AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
DualSpecFu.hasIcon = "Interface\\Icons\\Ability_Seal"
DualSpecFu.independentProfile = true
DualSpecFu.defaultPosition = "LEFT"
DualSpecFu.defaultMinimapPosition = 180
DualSpecFu.cannotDetachTooltip = false
DualSpecFu.tooltipType = "Tablet-2.0"
DualSpecFu.clickableTooltip = true
DualSpecFu:RegisterDB("DualSpecFuDB", "DualSpecFuDBPC")

DualSpecFu:RegisterDefaults("char", {new = 0, total = 0})
DualSpecFu:RegisterDefaults("profile", {
	filterSpam = true,
	showSpecName = true,
	currentNext = true,
	useCustomName = false,
	primSpecColor = {
		r = 1,
		g = 1,
		b = 0,
		a = 1,
	},
	secSpecColor = {
		r = 1,
		g = 1,
		b = 0,
		a = 1,
	},
	listalts = true,
})
DualSpecFu:RegisterDefaults("realm", {
	info = {},
})

function DualSpecFu:GetHexColor(class)
	return string.format("%02x%02x%02x", 
		RAID_CLASS_COLORS[class].r * 255,
		RAID_CLASS_COLORS[class].g * 255,
		RAID_CLASS_COLORS[class].b * 255
	);
end

local spamFilterMatch1 = string.gsub(ERR_LEARN_ABILITY_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch2 = string.gsub(ERR_LEARN_SPELL_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch3 = string.gsub(ERR_SPELL_UNLEARNED_S:gsub('%.', '%.'), '%%s', '(.*)')

local GetNumTalentGroups, GetActiveTalentGroup = GetNumTalentGroups, GetActiveTalentGroup

function DualSpecFu:OnMenuRequest(level, value, inTooltip)
	local showSpecName = DualSpecFu:IsShowName()
	local showCustomName = DualSpecFu:IsCustomName()
	local listEmpty1
	local listEmpty2
	
	for i = 1, GetNumTalentGroups() do
		specCache[i] = specCache[i] or {}
		local thisCache = specCache[i]
		TalentFrame_UpdateSpecInfoCache(thisCache, false, false, i)
		if thisCache.primaryTabIndex and thisCache.primaryTabIndex ~= 0 then
			thisCache.specName = thisCache[thisCache.primaryTabIndex].name
			thisCache.mainTabIcon = thisCache[thisCache.primaryTabIndex].icon
		else
			thisCache.specName = "|cffff0000Talents undefined!|r"
			thisCache.mainTabIcon = "Interface\\Icons\\Ability_Seal"
		end
		thisCache.specGroupName = groupNamesCaps[i]
	end
	
	local specIcon1 = "Interface\\Icons\\Ability_Seal"
	local specIcon2

	iconCache[1] = iconCache[1] or {}
	local specIconCache1 = iconCache[1]
	TalentFrame_UpdateSpecInfoCache(specIconCache1, false, false, 1)
	if specIconCache1.primaryTabIndex and specIconCache1.primaryTabIndex ~= 0 then
		specIconCache1.tabIcon = specIconCache1[specIconCache1.primaryTabIndex].icon
	else
		specIconCache1.tabIcon = "Interface\\Icons\\Ability_Seal"
	end
	
	iconCache[2] = iconCache[2] or {}
	local specIconCache2 = iconCache[2]
	TalentFrame_UpdateSpecInfoCache(specIconCache2, false, false, 2)
	if specIconCache2.primaryTabIndex and specIconCache2.primaryTabIndex ~= 0 then
		specIconCache2.tabIcon = specIconCache2[specIconCache2.primaryTabIndex].icon
	else
		specIconCache2.tabIcon = "Interface\\Icons\\Ability_Seal"
	end
	
	local activeGroupNum = GetActiveTalentGroup()
	local curCache = specCache[activeGroupNum]
	local nextGroup = -activeGroupNum + 3
	local nextCache = specCache[nextGroup]
	
	local groupColoringNames = {
		self:TextColorFormat(self.db.profile.primSpecColor),
		self:TextColorFormat(self.db.profile.secSpecColor)
		}
	
	if showSpecName then
	    listEmpty1 = false
		if showCustomName then
	    	listEmpty2 = false
		else
			self.db.profile.useCustomName = false
	    	listEmpty2 = true
		end
	else
	    listEmpty1 = true
	    self.db.profile.useCustomName = false
	    listEmpty2 = true
	end

	if (level == 1) then
	    dewdrop:AddLine(
     		'text', "Open Talents",
   			'tooltipTitle', "Open Talents",
            'tooltipText', "Open main Talents window",
            'icon', curCache.mainTabIcon,
            'func', function() ToggleTalentFrame(); end
		)
		dewdrop:AddLine()
		dewdrop:AddLine(
  			'text', "Hide switch spam",
  			'tooltipTitle', "Hide switch spam",
			'tooltipText', "Hides the spell learned/unlearned messages when swapping specs.",
			'checked', self.db.profile.filterSpam,
			'func', function() self:ToggleHideSpam() end
		)
		dewdrop:AddLine(
  			'text', "Toggle current/next",
  			'tooltipTitle', "Toggle current/next",
			'tooltipText', "Toggle showing current spec or spec that it will be changed to.",
			'checked', self.db.profile.currentNext,
			'func', function() self:ToggleCurr() end
		)
		dewdrop:AddLine()
		dewdrop:AddLine(
		    'text', "Spec naming options",
		    'tooltipTitle', "Spec naming options",
		    'tooltipText', "Advanced options for spec naming",
		    'icon', curCache.mainTabIcon,
		    'hasArrow', true,
		    'value', "advanced"
		)
		dewdrop:AddLine()
    elseif (level == 2) then
	    if (value == "advanced") then
			dewdrop:AddLine(
	  			'text', "Show spec name",
	  			'tooltipTitle', "Show spec name",
				'tooltipText', "Displays the name of the specialization instead of primary/secondary."..newLine..newLine.."|cffff0000Required for custom names!|r"..newLine,
				'checked', self.db.profile.showSpecName,
				'func', function() self:ToggleShowName() end
			)
			dewdrop:AddLine(
	  			'text', "Use custom names",
	  			'tooltipTitle', "Use custom names",
				'tooltipText', "Toggle displaying custom name of the specializations."..newLine..newLine.."|cffff0000Required for custom names!|r"..newLine,
				'checked', self.db.profile.useCustomName,
				'func', function() self:ToggleCustomName() end,
				'disabled', listEmpty1
			)
			dewdrop:AddLine()
			dewdrop:AddLine(
				'text', "Primary spec name",
				'tooltipTitle', "Primary spec name",
				'tooltipText', "Type in the desired name for your primary spec.",
				'icon', specIconCache1.tabIcon,
				'hasArrow', true,
				'hasEditBox', true,
				'editBoxText', self.db.profile.primSpecName,
				'editBoxFunc', function(primSpecName)
					self.db.profile.primSpecName = primSpecName
					self:Print(("Primary spec name changed to: "..groupColoringNames[1]..primSpecName.."|r"))
					self:Update()
					end,
				'disabled', listEmpty2
			)
			dewdrop:AddLine(
    			'text', "Primary spec color",
    			'tooltipTitle', "Primary spec color",
    			'tooltipText', "Color of the primary spec name.",
    			'hasColorSwatch', true,
    			'hasOpacity', true,
    			'r', self.db.profile.primSpecColor.r,
    			'g', self.db.profile.primSpecColor.g,
    			'b', self.db.profile.primSpecColor.b,
    			'a', self.db.profile.primSpecColor.a,
    			'colorFunc', function(r, g, b, a)
    				self:SetPrimSpecColor(r, g, b, a)
    				end,
    			'disabled', listEmpty2
			)
			dewdrop:AddLine(
				'text', "Secondary spec name",
				'tooltipTitle', "Secondary spec name",
				'tooltipText', "Type in the desired name for your secondary spec.",
				'icon', specIconCache2.tabIcon,
				'hasArrow', true,
				'hasEditBox', true,
				'editBoxText', self.db.profile.secSpecName,
				'editBoxFunc', function(secSpecName)
					self.db.profile.secSpecName = secSpecName
					self:Print(("Secondary spec name changed to: "..groupColoringNames[2]..secSpecName.."|r"))
					self:Update()
					end,
				'disabled', listEmpty2
			)
			dewdrop:AddLine(
    			'text', "Secondary spec color",
    			'tooltipTitle', "Secondary spec color",
    			'tooltipText', "Color of the secondary spec name.",
    			'hasColorSwatch', true,
    			'hasOpacity', true,
    			'r', self.db.profile.secSpecColor.r,
    			'g', self.db.profile.secSpecColor.g,
    			'b', self.db.profile.secSpecColor.b,
    			'a', self.db.profile.secSpecColor.a,
    			'colorFunc', function(r, g, b, a)
    				self:SetSecSpecColor(r, g, b, a)
    				end,
    			'disabled', listEmpty2
			)
		end
	end
end

function DualSpecFu:OnInitialize()
	self:SetIcon("Interface\\Icons\\Ability_Seal")
end

function DualSpecFu:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
end

function DualSpecFu:OnDisable()
	
end

local function filter(self, event, ...)
	local msg = arg1;
	local hideSpam = DualSpecFu:IsHideSpam()
	if hideSpam then
  		if strfind(msg, spamFilterMatch1) then
    		return true
		elseif strfind(msg, spamFilterMatch2) then
    		return true
		elseif strfind(msg, spamFilterMatch3) then
    		return true
		end
	end
end

function DualSpecFu:OnTextUpdate()
	if self:IsTextShown() then
        self:SetText(" |cffffffffDualSpec|cff00ff00Fu|r")
    else
    	self:HideText()
    end
    if not self:IsIconShown() then
        self:HideIcon()
    end
    self:OnUpdate()
end

function DualSpecFu:PLAYER_LEAVING_WORLD(event)
	self:UnregisterEvent("PLAYER_TALENT_UPDATE")
end

function DualSpecFu:PLAYER_ENTERING_WORLD(event)
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
end

function DualSpecFu:ACTIVE_TALENT_GROUP_CHANGED(event, talGroup)
    local showSpecName = DualSpecFu:IsShowName()
    local useCustomName = DualSpecFu:IsCustomName()
    
    local customPrimName = DualSpecFu:ValuePrimName()
	local customSecName = DualSpecFu:ValueSecName()
	
	local groupColoringNames = {
		self:TextColorFormat(self.db.profile.primSpecColor),
		self:TextColorFormat(self.db.profile.secSpecColor)
		}
	
	local customNameTable = {
	    customPrimName,
	    customSecName
	    }

	local newGroup = -talGroup + 3

	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	
	local a = specCache[newGroup][1].pointsSpent
	local b = specCache[newGroup][2].pointsSpent
	local c = specCache[newGroup][3].pointsSpent
	
    if showSpecName then
        if not useCustomName then
    		self:Print(("Switched to |cff6adb54"..specCache[newGroup].specName.." ("..a.."/"..b.."/"..c..")|r talent spec."))
		else
		    if customNameTable[newGroup] ~= nil then
		    	self:Print(("Switched to "..groupColoringNames[newGroup]..customNameTable[newGroup].." ("..a.."/"..b.."/"..c..")|r talent spec."))
			else
			    self:Print(("Switched to |cffff0000Name not set! |cff6adb54("..a.."/"..b.."/"..c..")|r talent spec."))
			end
		end
    else
		self:Print(("Switched to "..groupNames[newGroup].." talent spec."))
	end
	
end

function DualSpecFu:UNIT_SPELLCAST_START(event, ...)
	local unit, spellName = ...
	if unit == "player" and (spellName == primarySpecSpellName or spellName == secondarySpecSpellName) then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	end
end

function DualSpecFu:UNIT_SPELLCAST_INTERRUPTED(event, ...)
	local unit, spellName = ...
	if unit == "player" and (spellName == primarySpecSpellName or spellName == secondarySpecSpellName) then
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	end
end

function DualSpecFu:PLAYER_TALENT_UPDATE(event)
	self:OnUpdate()
end

function DualSpecFu:OnUpdate()

	local showSpecName = DualSpecFu:IsShowName()
	local useCustomName = DualSpecFu:IsCustomName()
	local toggleCurr = DualSpecFu:IsToggleCurr()
	
	local customPrimName = DualSpecFu:ValuePrimName()
	local customSecName = DualSpecFu:ValueSecName()
	
	local groupColoringNames = {
		self:TextColorFormat(self.db.profile.primSpecColor),
		self:TextColorFormat(self.db.profile.secSpecColor)
		}
	
	for i = 1, GetNumTalentGroups() do
		specCache[i] = specCache[i] or {}
		local thisCache = specCache[i]
		TalentFrame_UpdateSpecInfoCache(thisCache, false, false, i)
		if thisCache.primaryTabIndex and thisCache.primaryTabIndex ~= 0 then
			thisCache.specName = thisCache[thisCache.primaryTabIndex].name
			thisCache.mainTabIcon = thisCache[thisCache.primaryTabIndex].icon
		else
			thisCache.specName = "|cffff0000Talents undefined!|r"
			thisCache.mainTabIcon = "Interface\\Icons\\Ability_Seal"
		end
		thisCache.specGroupName = groupNamesCaps[i]
	end
	
	local activeGroupNum = GetActiveTalentGroup()
	local curCache = specCache[activeGroupNum]

	local customNameTable = {
	    customPrimName,
	    customSecName
	    }
	
	local a = curCache[1].pointsSpent
	local b = curCache[2].pointsSpent
	local c = curCache[3].pointsSpent
	
    if showSpecName then
        if not useCustomName then
	        if toggleCurr then
	        	if a <= 0 and b <= 0 and c <= 0 then
	            	DualSpecFu:SetText(" |cffff0000Talents undefined!|r")
	            	DualSpecFu:SetIcon(curCache.mainTabIcon)
				else
			    	if a + 1 == b or a - 1 == b or a + 1 == c or a - 1 == c or b + 1 == c or b - 1 == c then
						DualSpecFu:SetText(" Hybrid ("..a.."/"..b.."/"..c..")")
						DualSpecFu:SetIcon(curCache.mainTabIcon)
					else
						DualSpecFu:SetText(" "..curCache.specName.." ("..a.."/"..b.."/"..c..")")
						DualSpecFu:SetIcon(curCache.mainTabIcon)
					end
				end
			else
				if GetNumTalentGroups() >= 2 then
					local nextGroup = -activeGroupNum + 3
					local nextCache = specCache[nextGroup]
					
					local a3 = nextCache[1].pointsSpent
					local b3 = nextCache[2].pointsSpent
					local c3 = nextCache[3].pointsSpent
					
				    if a3 <= 0 and b3 <= 0 and c3 <= 0 then
				        DualSpecFu:SetText(" To: |cffff0000Talents undefined!|r")
				        DualSpecFu:SetIcon(nextCache.mainTabIcon)
					else
						if a3 + 1 == b3 or a3 - 1 == b3 or a3 + 1 == c3 or a3 - 1 == c3 or b3 + 1 == c3 or b3 - 1 == c3 then
							DualSpecFu:SetText(" To: Hybrid ("..a3.."/"..b3.."/"..c3..")")
							DualSpecFu:SetIcon(nextCache.mainTabIcon)
						else
							DualSpecFu:SetText(" To: "..nextCache.specName.." ("..a3.."/"..b3.."/"..c3..")")
							DualSpecFu:SetIcon(nextCache.mainTabIcon)
						end
					end
				else
					DualSpecFu:SetText(" To: |cffff0000Talents undefined!|r")
					DualSpecFu:SetIcon("Interface\\Icons\\Ability_Seal")
				end
			end
		else
		    if toggleCurr then
	        	if a <= 0 and b <= 0 and c <= 0 then
	            	DualSpecFu:SetText(" |cffff0000Talents undefined!|r")
	            	DualSpecFu:SetIcon(curCache.mainTabIcon)
				else
				    if customNameTable[activeGroupNum] ~= nil then
        				DualSpecFu:SetText(" "..groupColoringNames[activeGroupNum]..customNameTable[activeGroupNum].."|r ("..a.."/"..b.."/"..c..")")
						DualSpecFu:SetIcon(curCache.mainTabIcon)
					else
					    DualSpecFu:SetText(" |cffff0000Name not set!|r ("..a.."/"..b.."/"..c..")")
					    DualSpecFu:SetIcon(curCache.mainTabIcon)
					end
				end
			else
				if GetNumTalentGroups() >= 2 then
					local nextGroup = -activeGroupNum + 3
					local nextCache = specCache[nextGroup]
					
					local a3 = nextCache[1].pointsSpent
					local b3 = nextCache[2].pointsSpent
					local c3 = nextCache[3].pointsSpent
					
				    if a3 <= 0 and b3 <= 0 and c3 <= 0 then
				        DualSpecFu:SetText(" To: |cffff0000Talents undefined!|r")
				        DualSpecFu:SetIcon(nextCache.mainTabIcon)
					else
					    if customNameTable[nextGroup] ~= nil then
		     				DualSpecFu:SetText(" To: "..groupColoringNames[nextGroup]..customNameTable[nextGroup].."|r ("..a3.."/"..b3.."/"..c3..")")
							DualSpecFu:SetIcon(nextCache.mainTabIcon)
						else
						    DualSpecFu:SetText(" To: |cffff0000Name not set!|r ("..a3.."/"..b3.."/"..c3..")")
						    DualSpecFu:SetIcon(nextCache.mainTabIcon)
						end
					end
				else
					DualSpecFu:SetText(" To: |cffff0000Talents undefined!|r")
					DualSpecFu:SetIcon("Interface\\Icons\\Ability_Seal")
				end
			end
		end
	else
	    if toggleCurr then
			DualSpecFu:SetText(" "..curCache.specGroupName)
			DualSpecFu:SetIcon(curCache.mainTabIcon)
		else
			if GetNumTalentGroups() >= 2 then
				local nextGroup = -activeGroupNum + 3
				local nextCache = specCache[nextGroup]
				
				DualSpecFu:SetText(" To: "..nextCache.specGroupName)
				DualSpecFu:SetIcon(nextCache.mainTabIcon)
			else
				DualSpecFu:SetText(" To: |cffff0000Talents undefined!|r")
				DualSpecFu:SetIcon("Interface\\Icons\\Ability_Seal")
			end
		end
	end
	self:SaveInfo()
end

function DualSpecFu:OnClick(button)

	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	
	button = button or "LeftButton"
	
	if button == "LeftButton" then
		if GetNumTalentGroups() < 2 then
			DualSpecFu:Print("This character does not have dual specialization.")
			return
		end
		
		local curSpec = GetActiveTalentGroup()
		local newSpec = -curSpec + 3
		
		SetActiveTalentGroup(newSpec)
	end
end

function DualSpecFu:GetDropAlts()
	if self.db.profile.listalts then
		self.db.profile.listalts = false
	else
		self.db.profile.listalts = true
	end
	self:Update()
end

function DualSpecFu:OnTooltipUpdate()
	
	local cat = tablet:AddCategory(                                                      
		'columns', 2
	)
	cat:AddLine(
		'text', " "
	)

    for player, playerinfo in pairs(self.db.realm.info) do
		if playerinfo ~= nil and playerinfo ~= {} then
    
			if self.db.realm.info[player] ~= nil then

				local info = self.db.realm.info[player]
				local currPlayer = UnitName("player")
                
				if info.player == currPlayer then
					local c = cat:AddCategory(
						'columns', 2,
						'hideBlankLine', true,
						'text', "|cff"..self:GetHexColor(info.engClass).."["..info.player.."]|r",
						'text2', "|cff"..self:GetHexColor(info.engClass).."Lvl."..info.level.." "..className[info.engClass].."|r"
					)
	
					local primText = info.specs.primSpecText
					local secText = info.specs.secSpecText
					if primText ~= nil then
						c:AddLine(
							'text', " Primary spec:|r",
							'text2', primText
						)
					end
					if secText ~= nil then
						c:AddLine(
							'text', " Secondary spec:|r",
							'text2', secText
						)
					end
					c:AddLine(
						'text', " "
					)
					c:AddLine(
						'text', " "
					)
				end
			end	
		end
	end
	cat:AddLine(
		'text', "|cffffffffOther characters:"..endColor,
		'arg1', self,
		'func', "GetDropAlts",
		'arg2', self.db.profile.listalts
	)
	if self.db.profile.listalts then
	    for player, playerinfo in pairs(self.db.realm.info) do
			if playerinfo ~= nil and playerinfo ~= {} then

				if self.db.realm.info[player] ~= nil then

					local info = self.db.realm.info[player]
					local currPlayer = UnitName("player")

					if info.player ~= currPlayer then
						local c = cat:AddCategory(
							'columns', 2,
							'hideBlankLine', true,
							'text', "  |cff"..self:GetHexColor(info.engClass).."["..info.player.."]|r",
							'text2', "|cff"..self:GetHexColor(info.engClass).."Lvl."..info.level.." "..className[info.engClass].."|r"
						)

						local primText = info.specs.primSpecText
						local secText = info.specs.secSpecText
						if primText ~= nil then
							c:AddLine(
								'text', "    Primary spec:|r",
								'text2', primText
							)
						end
						if secText ~= nil then
							c:AddLine(
								'text', "    Secondary spec:|r",
								'text2', secText
							)
						end
					end
				end
			end
		end
	end
	
	
	
	local hint = ""
	local clickColor = "|cffeda55f"
	if GetNumTalentGroups() >= 2 then
		hint = hint..newLine..format("%s to switch spec.", clickColor.."Left click"..endColor)
				   ..newLine..format("%s to open Options menu.", clickColor.."Right click"..endColor)
				   ..newLine..format("%s \"Other characters:\" to show or hide the list.", clickColor.."Left click"..endColor)
	else
        hint = hint..newLine..format("%s to open Options menu.", clickColor.."Right click"..endColor)
                   ..newLine..format("%s \"Other characters:\" to show or hide the list.", clickColor.."Left click"..endColor)
	end
	
	tablet:SetHint(hint)
end

function DualSpecFu:SaveInfo()
	if (UnitLevel("player") < 40) then
		return;
	end

	local player = UnitName("player")
	if self.db.realm.info[player] == nil then
		self.db.realm.info[player] = {}
	end
	
	self.db.realm.info[player].player	= player
	self.db.realm.info[player].engClass	= select(2, UnitClass("player"))
	self.db.realm.info[player].level	= tostring(UnitLevel("player"))
	
	self.db.realm.info[player].specs	= {}

	---------------------------------------------------------------------------------------------------------

	local curGroup = GetActiveTalentGroup()
	local showSpecName = DualSpecFu:IsShowName()
	local useCustomName = DualSpecFu:IsCustomName()
	local curCache = specCache[curGroup]
	
	local customPrimName = DualSpecFu:ValuePrimName()
	local customSecName = DualSpecFu:ValueSecName()
	
	local groupColoringNames = {
		self:TextColorFormat(self.db.profile.primSpecColor),
		self:TextColorFormat(self.db.profile.secSpecColor)
		}
	
	if not curCache then
		self:OnUpdate()
		curCache = specCache[curGroup]
		if not curCache then
			local cat = tablet:AddCategory(
			'columns', 2 
			)
			cat:AddLine(
			'text', " Something realy bad happened"
			)
			return
		end
	end
	
	if not specCache[1][1] or specCache[1][1].pointsSpent == nil then
		return
	end

	local primSpecColor = "ffff99"
	local noPointsColor =  "|cffff0000"
	
	if curGroup == 1 then
		primSpecColor = "6adb54"
	end
	
	local customNameTable = {
	    customPrimName,
	    customSecName
	    }

	local a1 = specCache[1][1].pointsSpent
	local b1 = specCache[1][2].pointsSpent
	local c1 = specCache[1][3].pointsSpent
	
	local spec = {};

	if showSpecName then
		if a1 <= 0 and b1 <= 0 and c1 <= 0 then
			spec.primSpecText = noPointsColor.."No points spent!|r"
		else
		    if not useCustomName then
				if a1 + 1 == b1 or a1 - 1 == b1 or a1 + 1 == c1 or a1 - 1 == c1 or b1 + 1 == c1 or b1 - 1 == c1 then
					spec.primSpecText = "|cff"..primSpecColor.."Hybrid ("..a1.."/"..b1.."/"..c1..")|r"
				else
					spec.primSpecText = "|cff"..primSpecColor..specCache[1].specName.." ("..a1.."/"..b1.."/"..c1..")|r"
				end
				
				if GetNumTalentGroups() >= 2 then
		    	    local secSpecColor = "ffff99"
					if curGroup == 2 then
						secSpecColor = "6adb54"
					end
	
					local a2 = specCache[2][1].pointsSpent
					local b2 = specCache[2][2].pointsSpent
					local c2 = specCache[2][3].pointsSpent
	
		            if a2 <= 0 and b2 <= 0 and c2 <= 0 then
		            	spec.secSpecText = noPointsColor.."No points spent!|r"
					else
						if a2 + 1 == b2 or a2 - 1 == b2 or a2 + 1 == c2 or a2 - 1 == c2 or b2 + 1 == c2 or b2 - 1 == c2 then
							spec.secSpecText = "|cff"..secSpecColor.."Hybrid ("..a2.."/"..b2.."/"..c2..")|r"
						else
							spec.secSpecText = "|cff"..secSpecColor..specCache[2].specName.." ("..a2.."/"..b2.."/"..c2..")|r"
						end
					end
		    	end
		    else
			    if customPrimName ~= nil then
				    spec.primSpecText = groupColoringNames[1]..customPrimName.."|r |cff"..primSpecColor.."("..a1.."/"..b1.."/"..c1..")|r"
	
					if customSecName ~= nil then
						if GetNumTalentGroups() >= 2 then
				    	    local secSpecColor = "ffff99"
							if curGroup == 2 then
								secSpecColor = "6adb54"
							end
	
							local a2 = specCache[2][1].pointsSpent
							local b2 = specCache[2][2].pointsSpent
							local c2 = specCache[2][3].pointsSpent
	
							spec.secSpecText = groupColoringNames[2]..customSecName.."|r |cff"..secSpecColor.."("..a2.."/"..b2.."/"..c2..")|r"
						end
					else
					    if GetNumTalentGroups() >= 2 then
				    	    local secSpecColor = "ffff99"
							if curGroup == 2 then
								secSpecColor = "6adb54"
							end
	
							local a2 = specCache[2][1].pointsSpent
							local b2 = specCache[2][2].pointsSpent
							local c2 = specCache[2][3].pointsSpent
	
							spec.secSpecText = "|cffff0000Name not set!|cff"..secSpecColor.." ("..a2.."/"..b2.."/"..c2..")|r"
						end
					end
				else
				    spec.primSpecText = "|cffff0000Name not set!|cff"..primSpecColor.." ("..a1.."/"..b1.."/"..c1..")|r"

					if customSecName ~= nil then
			            if GetNumTalentGroups() >= 2 then
				    	    local secSpecColor = "ffff99"
							if curGroup == 2 then
								secSpecColor = "6adb54"
							end
	
							local a2 = specCache[2][1].pointsSpent
							local b2 = specCache[2][2].pointsSpent
							local c2 = specCache[2][3].pointsSpent
	
							spec.secSpecText = groupColoringNames[newGroup]..customSecName.."|r |cff"..secSpecColor.."("..a2.."/"..b2.."/"..c2..")|r"
						end
					else
					    if GetNumTalentGroups() >= 2 then
				    	    local secSpecColor = "ffff99"
							if curGroup == 2 then
								secSpecColor = "6adb54"
							end
	
							local a2 = specCache[2][1].pointsSpent
							local b2 = specCache[2][2].pointsSpent
							local c2 = specCache[2][3].pointsSpent
	
							spec.secSpecText = "|cffff0000Name not set!|cff"..secSpecColor.." ("..a2.."/"..b2.."/"..c2..")|r"
						end
					end
				end
			end
		end
	else
		if a1 <= 0 and b1 <= 0 and c1 <= 0 then
		    spec.primSpecText = noPointsColor.."No points spent!|r"
		else
			spec.primSpecText = "|cff"..primSpecColor..a1.."/"..b1.."/"..c1.."|r"
			
		    if GetNumTalentGroups() >= 2 then
		    	local secSpecColor = "ffff99"
				if curGroup == 2 then
					secSpecColor = "6adb54"
				end
	
				local a2 = specCache[2][1].pointsSpent
				local b2 = specCache[2][2].pointsSpent
				local c2 = specCache[2][3].pointsSpent
	
		        if a2 <= 0 and b2 <= 0 and c2 <= 0 then
		            spec.secSpecText = noPointsColor.."No points spent!|r"
				else
					spec.secSpecText = "|cff"..secSpecColor..a2.."/"..b2.."/"..c2.."|r"
				end
			end
		end
	end
		
	---------------------------------------------------------------------------------------------------------
	
	self.db.realm.info[player].specs = spec
	
	table.sort(DualSpecFu.db.realm.info, function(a,b) return a.player < b.player; end)

	return true
end

function DualSpecFu:IsHideSpam()
	return self.db.profile.filterSpam
end

function DualSpecFu:ToggleHideSpam()
	self.db.profile.filterSpam = not self.db.profile.filterSpam
	self:Update()
end

function DualSpecFu:IsShowName()
	return self.db.profile.showSpecName
end

function DualSpecFu:ToggleShowName()
	self.db.profile.showSpecName = not self.db.profile.showSpecName
	self:Update()
end

function DualSpecFu:IsToggleCurr()
	return self.db.profile.currentNext
end

function DualSpecFu:ToggleCurr()
	self.db.profile.currentNext = not self.db.profile.currentNext
	self:Update()
end

function DualSpecFu:IsCustomName()
	return self.db.profile.useCustomName
end

function DualSpecFu:ToggleCustomName()
	self.db.profile.useCustomName = not self.db.profile.useCustomName
	self:Update()
end

function DualSpecFu:ValuePrimName()
	return self.db.profile.primSpecName
end

function DualSpecFu:ValueSecName()
	return self.db.profile.secSpecName
end

function DualSpecFu:SetPrimSpecColor(r, g, b, a)
	self.db.profile.primSpecColor.r = r
	self.db.profile.primSpecColor.g = g
	self.db.profile.primSpecColor.b = b
	self.db.profile.primSpecColor.a = a
	self:Update()
end

function DualSpecFu:SetSecSpecColor(r, g, b, a)
	self.db.profile.secSpecColor.r = r
	self.db.profile.secSpecColor.g = g
	self.db.profile.secSpecColor.b = b
	self.db.profile.secSpecColor.a = a
	self:Update()
end

------------------------------------------
-- Color converters
------------------------------------------
local function RGBAToHex(r, g, b, a)
	-- WoW r,g,b,a scale is 0.0-1.0 so multiply first
	r = r * 255
	g = g * 255
	b = b * 255
	a = a * 255
	return string.format("%02x%02x%02x%02x", a, r, g, b)
end

function DualSpecFu:TextColorFormat(c)
	if c ~= nil then
		return "|c" .. RGBAToHex(c.r, c.g, c.b, c.a)
	else
		return "|c00000000"
	end
end